"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onGlobalStatsLastSuccessfulRequest = exports.onApiStatsChange = exports.onApiSettingsChange = exports.connectionChange = exports.onApiStatusUpdate = exports.onRequestRetry = exports.onRequestResponse = exports.onRequestStart = exports.apiError = void 0;
var types_1 = require("../types/types");
var actions_1 = require("./actions");
var types_2 = require("../types/types");
var date_1 = require("../sagas/_utils/date");
var patchGlobalSettings = actions_1.apiActions.api.global.settings.patch;
var patchGlobalStats = actions_1.apiActions.api.global.stats.patch;
var patchRequest = actions_1.apiActions.api.requests.patch;
var patchStats = actions_1.apiActions.api.status.patch;
var apiError = function (request, response) {
    return {
        type: 'API_ERROR',
        request: request,
        response: response,
    };
};
exports.apiError = apiError;
var onRequestStart = function (request) {
    return patchRequest(request.meta.id, {
        startTS: date_1.timestamp(),
        status: types_2.ApiRequestStatus.RUNNING,
    });
};
exports.onRequestStart = onRequestStart;
var onRequestResponse = function (request, response) {
    var _a = request.startTS, startTS = _a === void 0 ? 0 : _a;
    var _b = response.data, data = _b === void 0 ? '' : _b;
    var responseTS = date_1.timestamp();
    var duration = responseTS - startTS;
    var responseSize = JSON.stringify(data).length;
    var change = {
        responseTS: responseTS,
        responseSize: responseSize,
        duration: duration,
    };
    if (response.isSuccess) {
        change.status = types_2.ApiRequestStatus.SUCCESS;
        change.result = types_1.RequestResult.SUCCESS;
        change.completedTS = responseTS;
        change.isCompleted = true;
    }
    else {
        change.status = types_2.ApiRequestStatus.ERROR;
        change.result = types_1.RequestResult.ERROR;
        change.errorType = response.errorType;
        change.errorStatus = response.status;
    }
    return patchRequest(request.meta.id, change);
};
exports.onRequestResponse = onRequestResponse;
var onRequestRetry = function (request) {
    request.retriesCount = (request.retriesCount || 0) + 1;
    return patchRequest(request.meta.id, {
        retriesCount: request.retriesCount,
    });
};
exports.onRequestRetry = onRequestRetry;
var onApiStatusUpdate = function (request, response) {
    var nodeName = request.nodeName, startTS = request.startTS, responseTS = request.responseTS, completedTS = request.completedTS, duration = request.duration, responseSize = request.responseSize, result = request.result;
    return patchStats({
        nodeName: nodeName,
        lastRequest: {
            startTS: startTS,
            responseTS: responseTS,
            completedTS: completedTS,
            duration: duration,
            responseSize: responseSize,
            result: result,
        },
    });
};
exports.onApiStatusUpdate = onApiStatusUpdate;
var connectionChange = function (nodeName, connectionStatus) {
    return patchStats({
        nodeName: nodeName,
        connectionStatus: connectionStatus,
    });
};
exports.connectionChange = connectionChange;
var onApiSettingsChange = function (settings) {
    return patchGlobalSettings(settings);
};
exports.onApiSettingsChange = onApiSettingsChange;
var onApiStatsChange = function (settings) {
    return patchGlobalStats(settings);
};
exports.onApiStatsChange = onApiStatsChange;
var onGlobalStatsLastSuccessfulRequest = function (settings) {
    return exports.onApiStatsChange({ lastSuccessfulRequestTS: date_1.timestamp() });
};
exports.onGlobalStatsLastSuccessfulRequest = onGlobalStatsLastSuccessfulRequest;
//# sourceMappingURL=quickActions.js.map