"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSagasReducersForStore = exports.generateApiReducersForStore = exports.apiStatus = exports.apiEndpointsConfig = exports.lastAction = exports.apiRequests = exports.sagas = exports.generateConfigReducer = void 0;
var redux_1 = require("redux");
var reducers_1 = require("redux-store-generator/lib/reducers");
var generateConfigReducer = function (configName) {
    var config = function (state, action) {
        var newState = {};
        switch (action.type) {
            case "SET_" + configName:
                newState = __assign({}, action.payload);
                if (state.nodeType) {
                    newState.nodeType = state.nodeType;
                }
                return newState;
            case "PATCH_" + configName:
                newState = __assign(__assign({}, state), action.payload);
                if (state.nodeType) {
                    newState.nodeType = state.nodeType;
                }
                return newState;
        }
    };
    var configs = function (state, action) {
        var newState = {}, newAction = {}, payload = {};
        switch (action.type) {
            case "SET_ALL_" + configName:
                return action.payload;
            case "SET_" + configName:
            case "PATCH_" + configName:
                newState = __assign({}, state);
                payload = action.payload || {};
                Object.keys(payload).forEach(function (key) {
                    newAction = __assign(__assign({}, action), { payload: payload[key] });
                    newState[key] = config(newState[key], newAction);
                });
                return newState;
            default:
                return state || {};
        }
    };
    return configs;
};
exports.generateConfigReducer = generateConfigReducer;
var saga = function (state, action) {
    switch (action.type) {
        case 'SET_SAGA':
            return action.payload;
        case 'PATCH_SAGA':
            return __assign(__assign({}, state), action.payload);
    }
};
var sagas = function (state, action) {
    var _a;
    var newState = {}, id;
    switch (action.type) {
        case 'SET_ALL_SAGAS':
            return action.payload;
        case 'SET_SAGA':
        case 'PATCH_SAGA':
            newState = __assign({}, state);
            id = (_a = action.payload) === null || _a === void 0 ? void 0 : _a.id;
            if (id) {
                newState[id] = saga(state[id], action);
            }
            return newState;
        default:
            return state || {};
    }
};
exports.sagas = sagas;
var apiRequest = function (state, action) {
    var payload = action.payload;
    var request = payload.request;
    switch (action.type) {
        case 'PATCH_REQUEST':
            // mutability, an anti-pattern in most cases,
            // allows us to avoid fetching the request after every change
            // this is useful as requests are highly volatile
            Object.keys(request).forEach(function (key) {
                state[key] = request[key];
            });
            return state;
    }
};
var apiRequests = function (state, action) {
    var payload = action.payload;
    var id = (payload || {}).id;
    var newState = [], index;
    switch (action.type) {
        case 'PUSH_REQUEST':
            return __spreadArray(__spreadArray([], state), [action.payload]);
        case 'PATCH_REQUEST':
            newState = __spreadArray([], state);
            index = state.findIndex(function (request) { return request.meta.id === id; });
            if (index >= 0) {
                newState[index] = apiRequest(newState[index], action);
            }
            return newState;
        case 'REMOVE_REQUEST':
            newState = __spreadArray([], state);
            index = state.findIndex(function (request) { return request.meta.id === id; });
            if (index >= 0) {
                newState.splice(index, 1);
            }
            return newState;
        case 'PURGE_COMPLETED':
            return state.filter(function (request) { return !request.isCompleted; });
        default:
            return state || {};
    }
};
exports.apiRequests = apiRequests;
// https://github.com/reduxjs/redux/issues/580#issuecomment-133188511
var lastAction = function (state, action) {
    return action;
};
exports.lastAction = lastAction;
var apiSettings = reducers_1.generateSingle('API_GLOBAL_SETTINGS');
var apiStats = reducers_1.generateSingle('API_GLOBAL_STATS');
exports.apiEndpointsConfig = exports.generateConfigReducer('API_ENDPOINTS_CONFIG'); // prettier-ignore
exports.apiStatus = exports.generateConfigReducer('API_STATUS');
var apiActionTypes = reducers_1.generateSingle('API_ACTION_TYPES');
var apiNodeTypes = reducers_1.generateSingle('API_NODE_TYPES');
var generateApiReducersForStore = function () {
    return {
        _api: redux_1.combineReducers({
            apiGlobalSettings: apiSettings,
            apiGlobalStats: apiStats,
            endpointsConfig: exports.apiEndpointsConfig,
            status: exports.apiStatus,
            actionTypes: apiActionTypes,
            nodeTypes: apiNodeTypes,
            requests: exports.apiRequests,
        }),
    };
};
exports.generateApiReducersForStore = generateApiReducersForStore;
var generateSagasReducersForStore = function () {
    return {
        _sagas: exports.sagas,
    };
};
exports.generateSagasReducersForStore = generateSagasReducersForStore;
//# sourceMappingURL=reducers.js.map