"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.partialStore = exports.myStoreState = exports.state = exports.storeStructure = void 0;
var redux_store_generator_1 = require("redux-store-generator");
var types_1 = require("../types/types");
exports.storeStructure = {
    appState: redux_store_generator_1.NodeType.SINGLE_NODE,
    products: redux_store_generator_1.NodeType.COLLECTION_NODE,
    logs: redux_store_generator_1.NodeType.QUEUE_NODE,
};
exports.state = {
    apiGlobalSettings: {
        beat: 3000,
        maxConcurrentRequests: 30,
        retryStrategy: types_1.RetryStrategy.X3_TIMES,
        delayBetweenRetries: 1000,
        verifyInternetConnection: true,
        verifyAfterXMillisOfFailures: 10 * 1000,
    },
    apiGlobalStats: {
        internetConnectionAvailable: true,
        lastSuccessfulRequestTS: 0,
    },
    sagas: {
        get: { isRunning: false },
        requests: { isRunning: false },
        save: { isRunning: false },
    },
    endpointsConfig: {
        appState: {
            nodeType: redux_store_generator_1.NodeType.SINGLE_NODE,
            connectionType: types_1.ConnectionType.REST,
        },
        products: {
            nodeType: redux_store_generator_1.NodeType.COLLECTION_NODE,
            connectionType: types_1.ConnectionType.REST,
            retryStrategy: types_1.RetryStrategy.X3_TIMES,
            requestsPerMinute: 3,
            pagination: true,
        },
        logs: {
            nodeType: redux_store_generator_1.NodeType.QUEUE_NODE,
            connectionType: types_1.ConnectionType.SOCKET,
        },
    },
    status: {
        appState: {
            connectionStatus: types_1.ConnectionStatus.IDLE,
        },
        products: {
            connectionStatus: types_1.ConnectionStatus.IDLE,
        },
        logs: {
            connectionStatus: types_1.ConnectionStatus.IDLE,
        },
    },
};
exports.myStoreState = {
    appState: {
        isLoading: true,
    },
    user: {},
    products: {
        '1': {
            id: '1',
            title: 'first product',
            price: 149.5,
            thumbnail: '//www.gravatar.com/avatar/669d08ca748c87f4a2fc168881da1fe4?s=64&d=identicon&r=PG',
            imageUrl: '//www.gravatar.com/avatar/25760c726742f26ade0aa87b9fb7c201?s=500&d=identicon&r=PG',
            color: 'gold',
            shippingDate: '2021-10-10',
        },
    },
    logs: [
        {
            id: '1',
        },
    ],
    chats: {
        '1': {
            id: '1',
            items: [],
        },
    },
};
exports.partialStore = {
    _api: {
        actionTypes: {
            PATCH_APPSTATE: { verb: 'patch', nodeName: 'appState' },
            PATCH_PRODUCT: {
                verb: 'patch',
                nodeName: 'products',
            },
            SET_LOG: {
                verb: 'set',
                nodeName: 'logs',
                isLocal: true,
            },
        },
        endpointsConfig: {
            appState: {
                nodeType: 'SINGLE_NODE',
                connectionType: 'NONE',
            },
            products: {
                nodeType: 'COLLECTION_NODE',
                connectionType: 'REST',
            },
            logs: {
                nodeType: 'QUEUE_NODE',
                connectionType: 'REST',
            },
            chats: {
                nodeType: 'GROUPED_LIST_NODE',
                connectionType: 'REST',
            },
        },
    },
};
exports.default = exports.state;
//# sourceMappingURL=store-state.js.map