"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSagasInitialState = exports.generateInitialState = void 0;
var redux_store_generator_1 = require("redux-store-generator");
var types_1 = require("./types/types");
var redux_store_generator_2 = require("redux-store-generator");
var DEFAULT_API_GLOBAL_SETTINGS = {
    beat: 300,
    maxConcurrentRequests: 30,
    retryStrategy: types_1.RetryStrategy.X3_TIMES,
    delayBetweenRetries: 1000,
    verifyInternetConnection: false,
    verifyAfterXMillisOfFailures: 10 * 1000,
};
var DEFAULT_API_GLOBAL_STATS = {
    internetConnectionAvailable: true,
};
var DEFAULT_ENDPOINT_CONFIG = {
    connectionType: types_1.ConnectionType.REST,
};
var DEFAULT_STATE = {
    connectionStatus: types_1.ConnectionStatus.IDLE,
};
var DEFAULT_SAGA_STATE = {
    isRunning: false,
};
var generateInitialState = function (storeState, apiConfigOverrides) {
    var output = {
        apiGlobalSettings: __assign({}, DEFAULT_API_GLOBAL_SETTINGS),
        apiGlobalStats: __assign({}, DEFAULT_API_GLOBAL_STATS),
        endpointsConfig: {},
        status: {},
        actionTypes: {},
        nodeTypes: {},
        requests: [],
    };
    var keys = Object.keys(storeState);
    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
        var key = keys_1[_i];
        var value = storeState[key];
        var type = redux_store_generator_2.nodeToType(value);
        output.status[key] = __assign({}, DEFAULT_STATE);
        var overrides = (apiConfigOverrides || {})[key];
        output.endpointsConfig[key] = __assign(__assign({ nodeType: type }, DEFAULT_ENDPOINT_CONFIG), overrides);
    }
    output.actionTypes = redux_store_generator_1.generateActionTypesDictionaryForStore(storeState);
    output.nodeTypes = redux_store_generator_2.analyzeStructure(storeState);
    return output;
};
exports.generateInitialState = generateInitialState;
var generateSagasInitialState = function (sagas) {
    var output = {};
    var keys = Object.keys(sagas);
    for (var _i = 0, keys_2 = keys; _i < keys_2.length; _i++) {
        var key = keys_2[_i];
        var saga = sagas[key];
        output[key] = __assign({ id: key, description: saga.description }, DEFAULT_SAGA_STATE);
    }
    return output;
};
exports.generateSagasInitialState = generateSagasInitialState;
//# sourceMappingURL=initialState.js.map