"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostApiActionBuilder = void 0;
var redux_store_generator_1 = require("redux-store-generator");
var globals_1 = require("../../globals");
var PostApiActionBuilder = /** @class */ (function () {
    function PostApiActionBuilder() {
        this.apiRequest = {};
        this.apiResponse = {};
        this.actions = {};
        this.actions = redux_store_generator_1.generateActionsForStore(globals_1.default.structure);
    }
    PostApiActionBuilder.prototype.withRequest = function (apiRequest) {
        this.apiRequest = apiRequest;
        return this;
    };
    PostApiActionBuilder.prototype.withResponse = function (apiResponse) {
        this.apiResponse = apiResponse;
        return this;
    };
    PostApiActionBuilder.prototype.build = function () {
        var action = {};
        var _a = this.apiRequest, nodeType = _a.nodeType, apiVerb = _a.apiVerb, nodeName = _a.nodeName, originalAction = _a.originalAction;
        var _b = this.apiResponse.data, data = _b === void 0 ? {} : _b;
        var payload = (originalAction || {}).payload;
        var id = (payload || {}).id;
        var actionBag = this.actions[nodeName];
        switch (nodeType + '_' + apiVerb) {
            case 'SINGLE_NODE_get':
                action = actionBag['setAll'](data);
                break;
            case 'QUEUE_NODE_get':
                action = actionBag['pushMany'](data);
                break;
            case 'COLLECTION_NODE_get':
                action = actionBag['setMany']({ items: data });
                break;
            case 'COLLECTION_NODE_add':
                action = actionBag['set'](data.id, data);
                break;
            case 'COLLECTION_NODE_delete':
                action = actionBag['delete'](id);
                break;
            case 'SINGLE_NODE_patch':
            case 'COLLECTION_NODE_patch':
                action = actionBag['patch'](id, payload);
                break;
            case 'QUEUE_NODE_push':
                action = actionBag['push'](payload);
                break;
            case 'QUEUE_NODE_pop':
            case 'QUEUE_NODE_clear':
            default:
                action = actionBag[apiVerb]();
                break;
        }
        return action;
    };
    return PostApiActionBuilder;
}());
exports.PostApiActionBuilder = PostApiActionBuilder;
//# sourceMappingURL=PostApiActionBuilder.js.map