"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StoreBuilder = exports.DEFAULT_STORE_OPTIONS = void 0;
var process_manager_1 = require("./../process-manager");
var redux_1 = require("redux");
var redux_saga_1 = require("redux-saga");
var process_manager_2 = require("./../process-manager");
var redux_devtools_extension_1 = require("redux-devtools-extension");
exports.DEFAULT_STORE_OPTIONS = {
    devTools: false,
};
var StoreBuilder = /** @class */ (function () {
    function StoreBuilder(name) {
        var _this = this;
        this.definition = {
            name: '',
            initialState: {},
            reducers: {},
            middlewares: [],
            sagas: [],
            options: {},
        };
        this.preMiddlewares = [];
        this.postMiddlewares = [];
        this.fireSagas = function (store) {
            _this.definition.sagas.forEach(function (sagaId) {
                var action = process_manager_1.startSaga({ sagaId: sagaId });
                store.dispatch(action);
            });
        };
        this.definition.name = name;
    }
    StoreBuilder.prototype.withInitialState = function (initialState) {
        this.definition.initialState = __assign(__assign({}, this.definition.initialState), initialState);
        return this;
    };
    StoreBuilder.prototype.withReducers = function (reducers) {
        this.definition.reducers = __assign(__assign({}, this.definition.reducers), reducers);
        return this;
    };
    StoreBuilder.prototype.withMiddlewares = function () {
        var middlewares = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            middlewares[_i] = arguments[_i];
        }
        this.definition.middlewares = __spreadArray(__spreadArray([], middlewares), this.definition.middlewares);
        return this;
    };
    StoreBuilder.prototype.withPreMiddlewares = function () {
        var middlewares = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            middlewares[_i] = arguments[_i];
        }
        this.preMiddlewares = middlewares;
        return this;
    };
    StoreBuilder.prototype.withPostMiddlewares = function () {
        var middlewares = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            middlewares[_i] = arguments[_i];
        }
        this.postMiddlewares = middlewares;
        return this;
    };
    StoreBuilder.prototype.withSagas = function () {
        var sagas = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            sagas[_i] = arguments[_i];
        }
        this.definition.sagas = __spreadArray(__spreadArray([], this.definition.sagas), sagas);
        return this;
    };
    StoreBuilder.prototype.withOptions = function (options) {
        this.definition.options = options;
        return this;
    };
    Object.defineProperty(StoreBuilder.prototype, "options", {
        get: function () {
            return __assign(__assign({}, exports.DEFAULT_STORE_OPTIONS), this.definition.options);
        },
        enumerable: false,
        configurable: true
    });
    StoreBuilder.prototype.build = function () {
        var _this = this;
        var devTools = this.options.devTools;
        // add the saga middleware
        if (this.definition.sagas.length > 0) {
            this.sagaMiddleware = redux_saga_1.default();
            this.withMiddlewares(this.sagaMiddleware);
        }
        var initialState = this.definition.initialState;
        var rootReducer = redux_1.combineReducers(this.definition.reducers);
        var middlewareEnhancer = redux_1.applyMiddleware.apply(void 0, __spreadArray(__spreadArray(__spreadArray([], this.preMiddlewares), this.definition.middlewares), this.postMiddlewares));
        var composeMethod = devTools ? redux_devtools_extension_1.composeWithDevTools : redux_1.compose;
        var composedEnhancers = composeMethod(middlewareEnhancer);
        var store = redux_1.createStore(rootReducer, initialState, composedEnhancers);
        var processManager = new process_manager_2.default();
        if (this.definition.sagas.length > 0) {
            this.sagaMiddleware.run(processManager.run);
            setTimeout(function () { return _this.fireSagas(store); });
        }
        return store;
    };
    return StoreBuilder;
}());
exports.StoreBuilder = StoreBuilder;
//# sourceMappingURL=StoreBuilder.js.map