"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkConnection = exports.fireRequest = exports.api = exports.configureAxiosInstance = exports.DEFAULT_API_ENDPOINTS_CONFIGURATION = exports.instance = void 0;
var ResponseBuilder_1 = require("./ResponseBuilder");
var axios_1 = require("axios");
exports.DEFAULT_API_ENDPOINTS_CONFIGURATION = {
    internetConnectionCheckUrl: 'https://www.google.com',
};
var configureAxiosInstance = function (apiServerConfiguration) {
    var configuration = __assign(__assign({}, exports.DEFAULT_API_ENDPOINTS_CONFIGURATION), apiServerConfiguration);
    exports.instance = axios_1.default.create({
        baseURL: configuration.baseURL,
        timeout: configuration.timeout,
        headers: configuration.headers,
    });
};
exports.configureAxiosInstance = configureAxiosInstance;
exports.api = {
    get: function (path, params) { return exports.instance.get(path, { params: params }); },
    post: function (path, data) { return exports.instance.post(path, data); },
    patch: function (path, data) { return exports.instance.patch(path, data); },
    put: function (path, data) { return exports.instance.put(path, data); },
    delete: function (path) { return exports.instance.delete(path); },
};
var fireRequest = function (request) {
    var method = request.method, path = request.path, params = request.params;
    var response = new ResponseBuilder_1.ResponseBuilder(request);
    return new Promise(function (resolve) {
        var reqPromise;
        switch (method) {
            case 'GET':
                reqPromise = exports.api.get(path, params);
                break;
            case 'POST':
                reqPromise = exports.api.post(path, params);
                break;
            case 'PATCH':
                reqPromise = exports.api.patch(path, params);
                break;
            case 'DELETE':
                reqPromise = exports.api.delete(path);
                break;
            case 'PUT':
                reqPromise = exports.api.put(path, params);
                break;
        }
        reqPromise
            .then(function (res) {
            response.withIsSuccess(true).withAxiosResponse(res);
        })
            .catch(function (error) {
            if (error.response) {
                // The request was made and the server responded with a status code
                // that falls out of the range of 2xx
                var status_1 = error.response.status;
                var errorType = status_1 === 401 ? 'authorization' : 'server';
                response
                    .withErrorType(errorType)
                    .withAxiosResponse(error.response);
            }
            else if (error.request) {
                // The request was made but no response was received
                // `error.request` is an instance of XMLHttpRequest in the browser and an instance of
                // http.ClientRequest in node.js
                response.withErrorType('timeout');
            }
            else {
                // Something happened in setting up the request that triggered an Error
                response
                    .withErrorType('javascript')
                    .withErrorMessage(error.message);
            }
        })
            .finally(function () {
            resolve(response.build());
        });
    });
};
exports.fireRequest = fireRequest;
var checkConnection = function (url) {
    return new Promise(function (resolve) {
        axios_1.default
            .get(url)
            .then(function () {
            resolve(true);
        })
            .catch(function () {
            resolve(false);
        });
    });
};
exports.checkConnection = checkConnection;
//# sourceMappingURL=api.js.map