"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateConnectedStore = exports.generateStore = void 0;
var api_1 = require("./sagas/_utils/api");
var globals_1 = require("./globals");
var log_1 = require("./sagas/_utils/log");
var midConnected_1 = require("./middlewares/midConnected");
var reducers_1 = require("./connected/reducers");
var StoreBuilder_1 = require("./sagas/_utils/StoreBuilder");
var storeDefinitions_1 = require("./storeDefinitions");
var midFilter_1 = require("./middlewares/midFilter");
var CATCH_CONSOLE_LOG = false;
var generateStore = function (state, options) {
    globals_1.default.structure = state;
    var storeOptions = __assign(__assign({}, StoreBuilder_1.DEFAULT_STORE_OPTIONS), options);
    var mainStoreDefinition = storeDefinitions_1.getMainStoreDefinition(state, storeOptions); // prettier-ignore
    var mainStoreBuilder = new StoreBuilder_1.StoreBuilder('main');
    mainStoreBuilder
        .withInitialState(__assign({}, mainStoreDefinition.initialState))
        .withReducers(__assign({}, mainStoreDefinition.reducers))
        .withMiddlewares(midConnected_1.connectedMiddleware)
        .withOptions(options);
    globals_1.default.mainStore = mainStoreBuilder.build();
    api_1.configureAxiosInstance(storeOptions.apiServer);
    return globals_1.default.mainStore;
};
exports.generateStore = generateStore;
var generateConnectedStore = function (state, options) {
    var storeOptions = __assign(__assign({}, StoreBuilder_1.DEFAULT_STORE_OPTIONS), options);
    var connectedStoreDefinition = storeDefinitions_1.getConnectStoreDefinition(state, storeOptions); // prettier-ignore
    var connectedStoreBuilder = new StoreBuilder_1.StoreBuilder('connected');
    connectedStoreBuilder
        .withInitialState(__assign({}, connectedStoreDefinition.initialState))
        .withReducers(__assign(__assign({}, connectedStoreDefinition.reducers), { _lastAction: reducers_1.lastAction }))
        .withMiddlewares(midFilter_1.filterMiddleware)
        .withSagas('logger', 'requests', 'get', 'save', 'internet');
    globals_1.default.connectedStore = connectedStoreBuilder.build();
    return globals_1.default.connectedStore;
};
exports.generateConnectedStore = generateConnectedStore;
if (CATCH_CONSOLE_LOG) {
    log_1.captureLog(globals_1.default.mainStore);
}
/*
store.dispatch(apiActions.api.config.appState.patch({ requestsPerMinute: 4 }));
store.dispatch(apiActions.api.config.site.patch({ requestsPerMinute: 5 }));
store.dispatch(apiActions.api.config.products.patch({ requestsPerMinute: 6 }));

store.dispatch(apiActions.api.state.appState.patch({ connectionStatus: ConnectionStatus.LOADING })); // prettier-ignore
store.dispatch(apiActions.api.state.site.patch({ connectionStatus: ConnectionStatus.LOADING })); // prettier-ignore
store.dispatch(apiActions.api.state.products.patch({ connectionStatus: ConnectionStatus.LOADING })); // prettier-ignore

store.dispatch(apiActions.api.global.settings.patch({ verifyInternetConnection: true })); // prettier-ignore
store.dispatch(apiActions.api.global.stats.patch({ lastSuccessfulRequestTS: 1 })); // prettier-ignore
*/
//# sourceMappingURL=store.js.map