import { generateMeta } from './meta';
import { ApiResponse, ApiRequest, ApiErrorType } from '../../types/types';
import { AxiosResponse } from 'axios';

let sequence = 1;

export class ResponseBuilder {
    private output = {} as ApiResponse;
    private response = {} as AxiosResponse;

    constructor(request: ApiRequest) {
        this.output.meta = generateMeta(sequence++);
        this.output.request = request;
    }

    withAxiosResponse(axiosResponse: AxiosResponse) {
        this.response = axiosResponse;
        return this;
    }

    withIsSuccess(isSuccess: boolean) {
        this.output.isSuccess = isSuccess;
        return this;
    }

    withIsError(isError: boolean) {
        this.output.isError = isError;
        return this;
    }

    withErrorType(errorType: ApiErrorType) {
        this.output.errorType = errorType;
        return this;
    }

    withErrorMessage(errorMessage: string) {
        this.output.errorMessage = errorMessage;
        return this;
    }

    build(): ApiResponse {
        this.output.status = this.response.status;
        this.output.statusText = this.response.statusText;
        this.output.data = this.response.data;
        this.output.headers = this.response.headers;
        return this.output;
    }
}
