import { configureAxiosInstance } from './sagas/_utils/api';
import globals from './globals';
import { EndpointsConfigOverrides, StoreOptions } from './types/types';
import { captureLog } from './sagas/_utils/log';
import { connectedMiddleware } from './middlewares/midConnected';
import { lastAction } from './connected/reducers';
import { StoreStructure } from 'redux-store-generator';
import {
    StoreBuilder,
    DEFAULT_STORE_OPTIONS,
} from './sagas/_utils/StoreBuilder';
import {
    getConnectStoreDefinition,
    getMainStoreDefinition,
} from './storeDefinitions';
import { filterMiddleware } from './middlewares/midFilter';

const CATCH_CONSOLE_LOG = false;

export const generateStore = <T extends StoreStructure>(
    state: T,
    options: Partial<StoreOptions>
): any => {
    globals.structure = state;

    const storeOptions: StoreOptions = {
        ...DEFAULT_STORE_OPTIONS,
        ...options,
    } as StoreOptions;

    const mainStoreDefinition = getMainStoreDefinition(state, storeOptions); // prettier-ignore
    const mainStoreBuilder = new StoreBuilder('main');

    mainStoreBuilder
        .withInitialState({
            ...mainStoreDefinition.initialState,
        })
        .withReducers({
            ...mainStoreDefinition.reducers,
        })
        .withMiddlewares(connectedMiddleware)
        .withOptions(options);

    globals.mainStore = mainStoreBuilder.build();

    configureAxiosInstance(storeOptions.apiServer);

    return globals.mainStore;
};

export const generateConnectedStore = <T extends StoreStructure>(
    state: T,
    options?: Partial<StoreOptions>
): any => {
    const storeOptions: StoreOptions = {
        ...DEFAULT_STORE_OPTIONS,
        ...options,
    } as StoreOptions;

    const connectedStoreDefinition = getConnectStoreDefinition(state, storeOptions); // prettier-ignore
    const connectedStoreBuilder = new StoreBuilder('connected');

    connectedStoreBuilder
        .withInitialState({ ...connectedStoreDefinition.initialState })
        .withReducers({
            ...connectedStoreDefinition.reducers,
            _lastAction: lastAction,
        })
        .withMiddlewares(filterMiddleware)
        .withSagas('logger', 'requests', 'get', 'save', 'internet');

    globals.connectedStore = connectedStoreBuilder.build();

    return globals.connectedStore;
};

if (CATCH_CONSOLE_LOG) {
    captureLog(globals.mainStore);
}
/*
store.dispatch(apiActions.api.config.appState.patch({ requestsPerMinute: 4 }));
store.dispatch(apiActions.api.config.site.patch({ requestsPerMinute: 5 }));
store.dispatch(apiActions.api.config.products.patch({ requestsPerMinute: 6 }));

store.dispatch(apiActions.api.state.appState.patch({ connectionStatus: ConnectionStatus.LOADING })); // prettier-ignore
store.dispatch(apiActions.api.state.site.patch({ connectionStatus: ConnectionStatus.LOADING })); // prettier-ignore
store.dispatch(apiActions.api.state.products.patch({ connectionStatus: ConnectionStatus.LOADING })); // prettier-ignore

store.dispatch(apiActions.api.global.settings.patch({ verifyInternetConnection: true })); // prettier-ignore
store.dispatch(apiActions.api.global.stats.patch({ lastSuccessfulRequestTS: 1 })); // prettier-ignore
*/
